export declare class DependencyGraph<A> {
    private readonly nodes;
    private readonly _dependencies;
    constructor(nodes?: Record<string, A>, _dependencies?: Map<string, Set<string>>);
    /**
     * Return a single node in the graph
     */
    get(key: string): A;
    /**
     * Return a copy of the graph, restricted to only the nodes in the list
     */
    restrict(keys: string[]): DependencyGraph<A>;
    /**
     * The node is that are direct dependencies of the given node
     */
    directDependencies(key: string): string[];
    /**
     * The node is that are direct dependents on the given node
     */
    directDependents(key: string): string[];
    /**
     * All nodes that are dependencies of this node, and this node itself
     */
    upstream(key: string): DependencyGraph<A>;
    /**
     * All nodes that are dependents of this node, and this node itself
     */
    downstream(key: string): DependencyGraph<A>;
    /**
     * Turn the graph into a queue that can be consumed in dependency order
     */
    topoQueue(): TopoQueue<A>;
    /**
     * Whether the given node has dependencies
     */
    hasDependencies(key: string): boolean;
    /**
     * Remove a node from the graph
     */
    removeNode(key: string): void;
    /**
     * Set a node in the graph
     */
    setNode(key: string, node: A, deps?: Set<string>): void;
    /**
     * Merge two graphs, return a new graph
     */
    merge<B>(other: DependencyGraph<B>): DependencyGraph<A | B>;
    /**
     * Return a copy of the graph
     */
    copy(): DependencyGraph<A>;
    /**
     * List all keys
     */
    get keys(): Set<string>;
    /**
     * Access the dependency map
     */
    get dependencies(): ReadonlyMap<string, ReadonlySet<string>>;
    /**
     * Iterate over all elements in the graph in execution order
     */
    forEach<B>(block: (identifier: string, element: A) => B): void;
    /**
     * Map over all elements in the graph in execution order, without changing dependencies
     */
    map<B>(block: (identifier: string, element: A) => B): DependencyGraph<B>;
    private copyDependencies;
    private closure;
}
/**
 * A queue that will deliver its values in topologically sorted order
 */
export declare class TopoQueue<A> {
    private readonly blocked;
    private readonly available;
    private readonly originalGraph;
    private readonly currentGraph;
    constructor(graph: DependencyGraph<A>);
    isEmpty(): boolean;
    take(): QueueElement<A>;
    withNext<B>(block: (identifier: string, element: A) => B): B;
    peek(): PeekElements<A>;
    private advance;
    private determineAvailable;
}
export interface QueueElement<A> {
    readonly identifier: string;
    readonly element: A;
    remove(): void;
}
export interface PeekElements<A> {
    readonly elements: Array<readonly [string, A]>;
    skip(): void;
}
